import json
from io import StringIO

from samon.elements import BaseElement
from samon.render import RenderedElement


class KrApp(BaseElement):
    def to_xml(self, io: StringIO, indent: int, rendered_element: RenderedElement):
        with rendered_element.frame(io, indent):
            print('<script type="text/javascript">', file=io, end='')
            print(f"let ${self.xml_attrs['id']}_data = ", file=io, end='')
            data = rendered_element.to_json()
            json.dump(data, io)
            print('</script>', file=io)



import datetime

from django import forms
from django.forms import widgets
from django_kirui.widgets import CheckboxSwitch


class SampleForm(forms.Form):
    elso = forms.CharField(label='Próba felirat', required=False)
    harmadik = forms.MultipleChoiceField(label='Harmadik', choices=[(1, 'One'), (2, 'Two'), (3, 'Three')], initial=['1', '2'],
                                         widget=widgets.CheckboxSelectMultiple)
    masodik = forms.ChoiceField(label='Próba választó', choices=[(None, '-----------------'), (1, 'One'), (2, 'Two'), (3, 'Three')], initial=None)
    negyedik = forms.BooleanField(label='Valami', widget=CheckboxSwitch)
    otodik = forms.BooleanField(label='Ötödik', widget=CheckboxSwitch)
    hatodik = forms.IntegerField(label='Hatodik')
    hetedik = forms.DateField(label='Hetedik')  # , initial='2020-03-04')
    nyolcadik = forms.CharField(label='Nyolcadik', widget=widgets.Textarea)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['elso'].initial = str(datetime.datetime.now())



from browser import document
from django_brython.testing import location, reverse

from kirui.vdom.components import Component
from kirui.vdom.elements import Element


class ComponentTest:
    @location(reverse('backend:index'))
    def test_component_registered(self):
        el = document.createElement('custom-element')
        assert str(el) == '<HTMLElement>'
        assert getattr(el, 'custom_attribute', None) is None

        @Component(
            tag='custom-element',
            template=''
        )
        class CustomElement(Element):
            custom_attribute = 'test'

        el = document.createElement('custom-element')
        assert isinstance(el, CustomElement)
        assert getattr(el, 'custom_attribute', None) == 'test'

        document.body.appendChild(el)
        print('test finished')

    @location(reverse('backend:index'))
    def test_component_configuration(self):
        @Component(
            tag='custom-element',
            template='<div>Some content</div>'
        )
        class CustomElement:
            pass

        assert CustomElement._component_config.tag == 'custom-element'
        assert CustomElement._component_config.template.tagName == 'BODY'
        assert len(CustomElement._component_config.template.children) == 1
        assert CustomElement._component_config.template.children[0].tagName == 'DIV'
        assert CustomElement._component_config.template.children[0].textContent == 'Some content'
        print('test finished')



from unittest import TestCase
from mock import patch, MagicMock, mock_open
from livesync.fswatcher import FileWatcher, utils


class FileWatcherTestCase(TestCase):
    def setUp(self):
        self.watcher = FileWatcher()

    def test_add_new_handler(self):
        # act
        self.watcher.add_handler(1)
        # assert
        self.assertIn(1, self.watcher.handlers)

    @patch('livesync.fswatcher.watcher.Observer.start')
    @patch('livesync.fswatcher.watcher.Observer.schedule')
    def test_start_watching(self, mock_schedule, mock_start):
        # mock
        mock_handler = MagicMock()
        mock_handler.watched_paths=['.']
        self.watcher.handlers = set([mock_handler])
        #act
        self.watcher.start()
        # assert
        mock_start.assert_called()
        mock_schedule.assert_called_with(mock_handler, '.', recursive=True)

    @patch('livesync.fswatcher.watcher.Observer.stop')
    def test_stop_watching(self, mock_stop):
        #act
        self.watcher.stop()
        # assert
        mock_stop.assert_called()


class UtilsTestCase(TestCase):
    def test_get_md5_from_file(self):
        with patch('livesync.fswatcher.utils.open', mock_open(read_data=b'file content'), create=True):
            # act
            result = utils.get_md5("/path/file")
            # assert
            self.assertEqual(result, 'd10b4c3ff123b26dc068d43a8bef2d23')



from kirui.vdom.components import Component
from kirui.vdom.elements import Element

@Component(
    tag='kl-proba',
    template='12345678910sdsd',
    encapsulate_slot=True
)
class Proba(Element):
    def render(self):
        super().render()



from browser import document, window, webcomponent


class Component:
    slot_elements: dict

    def __init__(self, tag, template, encapsulate_slot=False):
        self.tag = tag

        parser = window.DOMParser.new()
        self.template = parser.parseFromString(template, 'text/html').body

        self.encapsulate_slot = encapsulate_slot

    def __call__(self, klass):
        setattr(klass, '_component_config', self)

        if webcomponent.get(self.tag) is None:
            webcomponent.define(self.tag, klass)

        return klass



from contextlib import contextmanager
from io import StringIO
from typing import Union, TYPE_CHECKING

from simpleeval import NameNotDefined

from . import constants

if TYPE_CHECKING:
    from .elements import BaseElement, AnonymusElement


class RenderedElement:
    node_attributes: dict

    def __init__(self, element: 'BaseElement', context: dict) -> None:
        self._element = element
        self._context = context
        self.node_attributes = self._eval_node_attributes(context)

    @property
    def node_name(self) -> str:
        return self._element.xml_tag

    def _eval_node_attributes(self, context) -> dict:
        retval = {}
        for k, v in self._element.xml_attrs.items():
            if k.startswith(f'{{{constants.XML_NAMESPACE_FLOW_CONTROL}}}'):
                continue
            elif k.startswith(f'{{{constants.XML_NAMESPACE_DATA_BINDING}}}'):
                k = k.replace(f'{{{constants.XML_NAMESPACE_DATA_BINDING}}}', '')

                retval[k] = v.eval(context)
            else:
                retval[k] = v

        return retval

    @property
    def children(self):
        if text := self._element.xml_attrs.get(f'{constants.XML_TEXT_BINDING_TAG}', None):
            yield text.eval(context=self._context)

        for child in self._element.children:
            klass = getattr(child.__class__, 'RENDERED_ELEMENT_CLASS', RenderedElement)

            if child.xml_tag is None:  # type: AnonymusElement
                yield child.text
            elif for_loop_def := child.xml_attrs.get(f'{{{constants.XML_NAMESPACE_FLOW_CONTROL}}}for', None):  # type: ForLoop
                for counter, loop_var_name, loop_var_val in for_loop_def.eval(self._context):
                    self._context['loop'] = {'index': counter, 'index0': counter - 1, 'odd': bool(counter % 2 == 1)}
                    self._context[loop_var_name] = loop_var_val

                    if_def = child.xml_attrs.get(f'{{{constants.XML_NAMESPACE_FLOW_CONTROL}}}if', None)
                    if if_def is None or if_def.eval(self._context):
                        yield klass(element=child, context=self._context)
            else:
                if_def = child.xml_attrs.get(f'{{{constants.XML_NAMESPACE_FLOW_CONTROL}}}if', None)
                if if_def is None or if_def.eval(self._context):
                    yield klass(element=child, context=self._context)

    def to_json(self):
        retval = [self.node_name, self.node_attributes, []]
        for child in self.children:
            if isinstance(child, str):
                retval[2].append(child)
            elif hasattr(child, 'to_json'):
                retval[2].append(child.to_json())
            else:
                retval[2].append(str(child))

        return retval

    @contextmanager
    def frame(self, io, indent):
        indent = constants.INDENT * indent

        xml_attrs = ''
        for k, v in self.node_attributes.items():
            xml_attrs += f' {k}="{v}"'

        prefix = '<!DOCTYPE HTML>' if self.node_name == 'html' else ''
        io.write(f'{indent}{prefix}<{self.node_name}{xml_attrs}>\n')
        yield
        io.write(f'{indent}</{self.node_name}>\n')

    def to_xml(self, io=None, indent=0):
        io = io or StringIO()
        with self.frame(io, indent):
            for child in self.children:
                if isinstance(child, str):
                    io.write(f'{constants.INDENT * (indent + 1)}{child}\n')
                else:
                    if hasattr(child._element, 'to_xml'):
                        child._element.to_xml(io, indent + 1, child)
                    else:
                        child.to_xml(io, indent + 1)

        return io.getvalue()

    def serialize(self, output='json'):
        if output == 'json':
            return self.to_json()
        elif output == 'xml':
            return self.to_xml()
        else:
            raise NotImplementedError(f'Invalid output format: {output}')
